#!/usr/bin/env python3
from bs4 import BeautifulSoup
from urllib.parse import quote_plus
import requests
import argparse

html_file = 'lemmy.ml-communities.html'
communities = []
processed_communities = []
cache_file = 'cache.txt'

# TODO
# Use the API to:
# - get sorted communities from remote instance
# - add communities to the local instance


def retrieve_processed_communities():
    try:
        with open(cache_file, 'r') as cache_file_handler:
            for c in cache_file_handler.readlines():
                processed_communities.append(c)
    except FileNotFoundError:
        return []


def save_processed_communities():
    with open(cache_file, 'w') as cache_file_handler:
        processed_communities.sort()
        cache_file_handler.writelines(processed_communities)


def list_remote_communities_from_url(lemmy_source_instance, lemmy_destination_instance):
    end_of_communities = False
    current_page = 1
    while not end_of_communities:
        print('Processing communities page {} for {}'.format(
            current_page, lemmy_source_instance))
        page_communities = []
        url = 'https://{}/communities/listing_type/Local/page/{}'.format(
            lemmy_source_instance, current_page)
        r = requests.get(url)
        if r.status_code != 200:
            raise Exception(
                'Did not receive 200 code from url: {}'.format(url))
        soup = BeautifulSoup(r.text, features="html.parser")
        for x in soup.find_all('a', attrs={'class': ''}):
            community = x.get('href')
            page_communities.append(community)
            communities.append(community)
        end_of_communities = len(page_communities) == 0
        current_page = current_page + 1
    print('Gathered {} communities'.format(len(communities)))
    communities.sort()


def list_remote_communities_from_file():
    with open(html_file, 'r') as html_file_handler:
        html_content = html_file_handler.read()
        soup = BeautifulSoup(html_content, features="html.parser")
        for x in soup.find_all('a', attrs={'class': ''}):
            communities.append(x.get('href'))
    print("Found {} communities".format(len(communities)))


def search_communities(communities_batch_size, lemmy_source_instance, lemmy_destination_instance, dry_run):
    processed_communities_count = 0
    for community in communities:
        if processed_communities_count >= communities_batch_size:
            print('Reached the number of processed communities limit({}), stopping...'.format(
                communities_batch_size))
            break
        community_url = 'https://{}{}'.format(lemmy_source_instance, community)
        if [c for c in processed_communities if c.rstrip() == community_url]:
            print("Already processed {}, skipping...".format(community_url))
            continue
        community_encoded_url = quote_plus(community_url)
        search_url = 'https://{}'.format(lemmy_destination_instance)
        search_url = search_url + '/search/q/{}'.format(community_encoded_url)
        search_url = search_url + '/type/All/sort/TopAll/listing_type/All/'
        search_url = search_url + 'community_id/0/creator_id/0/page/1'
        print("Searching for community {} on instance {} ({}/{})".format(
            community_url,
            lemmy_destination_instance,
            processed_communities_count + 1,
            communities_batch_size))
        if dry_run:
            print('Running in dry run mode, not doing anything')
            print("OK: Community {}".format(community_url))
            processed_communities_count = processed_communities_count + 1
            continue
        r = requests.get(search_url)
        if r.status_code != 200:
            print("ERROR: Could not search community {}, got status code {}".format(
                community_url, r.status_code))
        else:
            print("OK: Community {}".format(community_url))
            processed_communities.append("{}\n".format(community_url))
            processed_communities_count = processed_communities_count + 1
    save_processed_communities()


def main():
    parser = argparse.ArgumentParser(description="Search a Lemmy instance's communities from another one so as to get them listed")
    parser.add_argument(
        '--source',
        help='The instance communities are being pulled from ("lemmy.ml" for example)',
        action='store',
        required=True,
        dest='lemmy_source_instance')
    parser.add_argument(
        '--destination',
        help='The instance communities are being retrieved to ("mylemmy.com" for example)',
        action='store',
        required=True,
        dest='lemmy_destination_instance')
    parser.add_argument(
        '--communities-batch-size',
        help='Number of communities to pull on this batch',
        action='store',
        type=int,
        default=15,
        dest='communities_batch_size')
    parser.add_argument(
        '--no-dry-run',
        help='Set this flag to actually do the stuff',
        action='store_false',
        default=True,
        dest='dry_run')
    args = parser.parse_args()
    retrieve_processed_communities()
    list_remote_communities_from_url(
        lemmy_source_instance=args.lemmy_source_instance,
        lemmy_destination_instance=args.lemmy_destination_instance)
    search_communities(
        lemmy_source_instance=args.lemmy_source_instance,
        lemmy_destination_instance=args.lemmy_destination_instance,
        communities_batch_size=args.communities_batch_size,
        dry_run=args.dry_run)


if __name__ == "__main__":
    main()
